@@ -16,13 +16,14 @@ if gevent_socketio_found:
 import socketio
 import flask
 from flask import _request_ctx_stack, json as flask_json
+from flask.sessions import SessionMixin
 from werkzeug.debug import DebuggedApplication
 from werkzeug.serving import run_with_reloader
 
 from .namespace import Namespace
 from .test_client import SocketIOTestClient
 
-__version__ = '2.9.3'
+__version__ = '3.0.1'
 
 
 class _SocketIOMiddleware(socketio.Middleware):
@@ -42,6 +43,13 @@ class _SocketIOMiddleware(socketio.Middleware):
                                                          start_response)
 
 
+class _ManagedSession(dict, SessionMixin):
+    """This class is used for user sessions that are managed by
+    Flask-SocketIO. It is simple dict, expanded with the Flask session
+    attributes."""
+    pass
+
+
 class SocketIO(object):
     """Create a Flask-SocketIO server.
 
@@ -606,7 +614,7 @@ class SocketIO(object):
                 # created as a copy of the regular user session
                 if 'saved_session' not in self.server.environ[sid]:
                     self.server.environ[sid]['saved_session'] = \
-                        dict(flask.session)
+                        _ManagedSession(flask.session)
                 session_obj = self.server.environ[sid]['saved_session']
             else:
                 # let Flask handle the user session
@@ -825,7 +833,7 @@ def rooms(sid=None, namespace=None):
     return socketio.server.rooms(sid, namespace=namespace)
 
 
-def disconnect(silent=False):
+def disconnect(sid=None, namespace=None, silent=False):
     """Disconnect the client.
 
     This function terminates the connection with the client. As a result of
@@ -835,10 +843,16 @@ def disconnect(silent=False):
         def receive_message(msg):
             if is_banned(session['username']):
                 disconnect()
-            # ...
+            else:
+                # ...
 
+    :param sid: The session id of the client. If not provided, the client is
+                obtained from the request context.
+    :param namespace: The namespace for the room. If not provided, the
+                      namespace is obtained from the request context.
     :param silent: this option is deprecated.
     """
     socketio = flask.current_app.extensions['socketio']
-    return socketio.server.disconnect(flask.request.sid,
-                                      namespace=flask.request.namespace)
+    sid = sid or flask.request.sid
+    namespace = namespace or flask.request.namespace
+    return socketio.server.disconnect(sid, namespace=namespace)
